@@ -10,7 +10,7 @@ cgi.FieldStorage (without the bugs) and works with Python 2.5+ and 3.x (2to3).
 
 
 __author__ = "Marcel Hellkamp"
-__version__ = "0.2.2"
+__version__ = "0.2.3"
 __license__ = "MIT"
 __all__ = ["MultipartError", "MultipartParser", "MultipartPart", "parse_form_data"]
 
@@ -21,7 +21,7 @@ from io import BytesIO
 from tempfile import TemporaryFile
 from urllib.parse import parse_qs
 from wsgiref.headers import Headers
-from collections import MutableMapping as DictMixin
+from collections.abc import MutableMapping as DictMixin
 
 
 ##############################################################################
@@ -276,10 +276,13 @@ class MultipartParser(object):
         separator = b"--" + to_bytes(self.boundary)
         terminator = b"--" + to_bytes(self.boundary) + b"--"
 
-        # Consume first boundary. Ignore leading blank lines
+        # Consume first boundary. Ignore any preamble, as required by RFC
+        # 2046, section 5.1.1.
         for line, nl in lines:
-            if line:
+            if line in (separator, terminator):
                 break
+        else:
+            raise MultipartError("Stream does not contain boundary")
 
         # Check for empty data
         if line == terminator:
@@ -287,9 +290,6 @@ class MultipartParser(object):
                 raise MultipartError("Data after end of stream")
             return
 
-        if line != separator:
-            raise MultipartError("Stream does not start with boundary")
-
         # For each part in stream...
         mem_used, disk_used = 0, 0  # Track used resources to prevent DoS
         is_tail = False  # True if the last line was incomplete (cutted)
@@ -332,6 +332,11 @@ class MultipartParser(object):
                 except MultipartError:
                     part.close()
                     raise
+        else:
+            # If we run off the end of the loop, the current MultipartPart
+            # will not have been yielded, so it's our responsibility to
+            # close it.
+            part.close()
 
         if line != terminator:
             raise MultipartError("Unexpected end of multipart stream.")
